@@ -17,7 +17,7 @@ from devpi_common.vendor._pip import HTMLPage
 
 from devpi_common.url import URL
 from devpi_common.metadata import BasenameMeta
-from devpi_common.metadata import is_archive_of_project
+from devpi_common.metadata import is_archive_of_project, splitbasename
 from devpi_common.types import ensure_unicode_keys
 from devpi_common.validation import normalize_name, ensure_unicode
 from devpi_common.request import new_requests_session
@@ -204,7 +204,12 @@ class PyPIStage(BaseStage):
         from .model import ELink
         for relpath, md5, eggfragment in data:
             linkdict = dict(entrypath=relpath, md5=md5, eggfragment=eggfragment)
-            yield ELink(self.filestore, linkdict, projectname, "XXX")
+            version = "XXX"
+            try:
+                name, version = splitbasename(relpath)[:2]
+            except ValueError:
+                pass
+            yield ELink(self.filestore, linkdict, projectname, version)
 
     def clear_cache(self, projectname):
         normname = normalize_name(projectname)
@@ -234,6 +239,9 @@ class PyPIStage(BaseStage):
         threadlog.debug("visiting index %s", url)
         response = self.httpget(url, allow_redirects=True)
         if response.status_code != 200:
+            # XXX it's not correct to return UpstreamError in all cases
+            # if indeed the project was deleted but that fact
+            # is not yet properly processed
             raise self.UpstreamError("%s status on GET %s" %
                                      (response.status_code, url))
 
@@ -249,20 +257,23 @@ class PyPIStage(BaseStage):
             raise self.UpstreamError("no cache links from master for %s" %
                                      projectname)
 
-        # check returned url has the same normalized name
-        ret_projectname = response.url.strip("/").split("/")[-1]
-        assert normalize_name(projectname) == normalize_name(ret_projectname)
-
         # check that we got a fresh enough page
         serial = int(response.headers["X-PYPI-LAST-SERIAL"])
         newest_serial = self.pypimirror.name2serials.get(projectname, -1)
         if serial < newest_serial:
-            raise self.UpstreamError("%s: pypi returned serial %s, expected %s",
+            raise self.UpstreamError(
+                        "%s: pypi returned serial %s, expected %s",
                         projectname, serial, newest_serial)
 
         threadlog.debug("%s: got response with serial %s" %
                   (projectname, serial))
 
+
+        # check returned url has the same normalized name
+        ret_projectname = response.url.strip("/").split("/")[-1]
+        assert normalize_name(projectname) == normalize_name(ret_projectname)
+
+
         # parse simple index's link and perform crawling
         assert response.text is not None, response.text
         result = parse_index(response.url, response.text)
@@ -290,10 +301,7 @@ class PyPIStage(BaseStage):
         return versions
 
     def get_versiondata_perstage(self, projectname, version):
-        try:
-            links = self.get_releaselinks_perstage(projectname)
-        except self.UpstreamError:
-            return {}
+        links = self.get_releaselinks_perstage(projectname)
         verdata = {}
         for link in links:
             basename = link.basename
