@@ -209,9 +209,12 @@ class StatusView:
         }
         master_url = config.args.master_url
         if master_url:
+            from .replica import ReplicationErrors
             status["role"] = "REPLICA"
             status["master-url"] = master_url
             status["master-uuid"] = config.nodeinfo.get("master-uuid")
+            replication_errors = ReplicationErrors(self.xom.config.serverdir)
+            status["replication-errors"] = replication_errors.errors
         else:
             status["role"] = "MASTER"
         status["polling_replicas"] = self.xom.polling_replicas
@@ -737,6 +740,7 @@ class PyPIView:
         return view_verdata
 
     @view_config(route_name="/{user}/{index}/{name}/{version}",
+                 permission="del_verdata",
                  request_method="DELETE")
     def del_versiondata(self):
         stage = self.context.stage
@@ -770,7 +774,7 @@ class PyPIView:
             else:
                 abort(request, 410, "file existed, deleted in later serial")
 
-        if not entry.file_exists() or entry.eggfragment:
+        if should_fetch_remote_file(entry, request.headers):
             keyfs = self.xom.keyfs
             try:
                 if not self.xom.is_replica():
@@ -884,6 +888,16 @@ class PyPIView:
         apireturn(200, type="list:userconfig", result=d)
 
 
+def should_fetch_remote_file(entry, headers):
+    from .replica import H_REPLICA_FILEREPL
+    should_fetch = not entry.file_exists()
+    # if we are asked for an "egg" development link we cause
+    # refetching it unless we are called within file replication context
+    if entry.eggfragment and not headers.get(H_REPLICA_FILEREPL):
+        should_fetch = True
+    return should_fetch
+
+
 def url_for_entrypath(request, entrypath):
     parts = entrypath.split("/")
     user, index = parts[:2]
