@@ -198,7 +198,7 @@ def httpget(pypiurls):
             self.url2response = {}
             self._md5 = py.std.hashlib.md5()
 
-        def __call__(self, url, allow_redirects=False):
+        def __call__(self, url, allow_redirects=False, extra_headers=None):
             class mockresponse:
                 def __init__(xself, url):
                     fakeresponse = self.url2response.get(url)
@@ -649,6 +649,12 @@ class MyTestApp(TApp):
         assert r.status_code == code
         return r
 
+    def xdel(self, code, *args, **kwargs):
+        kwargs.setdefault("expect_errors", True)
+        r = self.delete(*args, **kwargs)
+        assert r.status_code == code
+        return r
+
 
     def get_json(self, *args, **kwargs):
         headers = kwargs.setdefault("headers", {})
