@@ -274,6 +274,7 @@ class TestStatus:
         data = r.json["result"]
         assert data["role"] == "REPLICA"
         assert data["serial"] == replica_xom.keyfs.get_current_serial()
+        assert data["replication-errors"] == {}
 
 
 def test_apiconfig_with_outside_url(testapp):
@@ -437,6 +438,12 @@ class TestSubmitValidation:
                 entry = testapp.xom.filestore.get_file_entry(path.strip("/"))
                 assert not entry.file_exists()
 
+    def test_delete_verdata_noacl_issue179(self, submit, testapp, mapp):
+        metadata = {"name": "pkg5", "version": "2.6", ":action": "submit"}
+        submit.metadata(metadata, code=200)
+        mapp.create_and_login_user("user2")
+        testapp.xdel(403, "/%s/pkg5/2.6" % submit.stagename)
+
     def test_upload_and_delete_user_issue130(self, submit, testapp, mapp):
         metadata = {"name": "pkg5", "version": "2.6", ":action": "submit"}
         submit.metadata(metadata, code=200)
